﻿using System;
using System.Text;
using System.Linq;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;
using System.Diagnostics;

namespace ScaraBattle.Persistence
{
    /// <summary>
    /// Save and load game state using files
    /// </summary>
    public class GameFileDataAccess : IGameDataAccess
    {
        /// <summary>
        /// Load game state from file
        /// </summary>
        /// <param name="path">Path of the file</param>
        /// <returns>Game state parameters</returns>
        public async Task<GameTable> LoadAsync(String path)
        {
            try
            {
                using (StreamReader reader = new StreamReader(path))
                {
                    int[] size = reader.ReadLine().Split(' ').Select(n => Int32.Parse(n)).ToArray();
                    String st = reader.ReadLine();
                    GameState state = st == "blue" ? GameState.Blue : (st == "red" ? GameState.Red : GameState.MapEditor);

                    GameTable table = new GameTable(size[0], size[1], state);

                    // Szükséges lehet tovább adni
                    // bool[,] _availablecells = new bool[table.Height, table.Width];
                    
                    int r = 0;
                    int c = 0;
                    for (int i = 0; i<table.Height; i++)
                    {
                        String[] row = reader.ReadLine().Split(' ');
                        foreach (String obj in row)
                        {
                            // _availablecells[_r, c] = false;
                            switch (obj)
                            {
                                case "D":
                                    table[r, c] = Dune.Instance;
                                    break;
                                case "O":
                                    table[r, c] = Oasis.Instance;
                                    break;
                                case "R":
                                    table[r, c] = Ruins.Instance;
                                    break;
                                default:
                                    // _availablecells[_r, c] = true;
                                    table[r, c] = null;
                                    break;
                            }
                            c++;
                        }
                        r++;
                        c = 0;
                    }

                    int castleNum = Int32.Parse(reader.ReadLine());
                    if (castleNum == 1)
                    {
                        int[] blueCastle;
                        blueCastle = reader.ReadLine().Split(' ').Select(n => Int32.Parse(n)).ToArray();
                        table.BlueCastle = new Castle() { IsBlue=true };
                        table[blueCastle[0], blueCastle[1]] = table.BlueCastle;
                        table.BlueCastle.Position = (blueCastle[0], blueCastle[1]);
                    }
                    castleNum = Int32.Parse(reader.ReadLine());
                    if (castleNum == 1)
                    {
                        int[] redCastle;
                        redCastle = reader.ReadLine().Split(' ').Select(n => Int32.Parse(n)).ToArray();
                        table.RedCastle = new Castle() { IsBlue=false };
                        table[redCastle[0], redCastle[1]] = table.RedCastle;
                        table.RedCastle.Position = (redCastle[0], redCastle[1]);
                    }
                    

                    int blueBarrack = Int32.Parse(reader.ReadLine());
                    if (blueBarrack != 0)
                    {
                        table.BlueBarracks = new (int, int)[blueBarrack];
                        int indexBlue = 0;
                        int[] coords = reader.ReadLine().Split(' ').Select(e => Int32.Parse(e)).ToArray();
                        for (int i = 0; i<coords.Length; i+=2)
                        {
                            table.BlueBarracks[indexBlue] = (coords[i], coords[i+1]);
                            table[coords[i], coords[i+1]] = new Barrack() { IsBlue = true };
                            indexBlue++;
                        }
                    }

                    int redBarrack = Int32.Parse(reader.ReadLine());
                    if (redBarrack != 0)
                    {
                        table.RedBarracks = new (int, int)[redBarrack];
                        int indexRed = 0;
                        int[] coords = reader.ReadLine().Split(' ').Select(e => Int32.Parse(e)).ToArray();
                        for (int i = 0; i < coords.Length; i += 2)
                        {
                            table.RedBarracks[indexRed] = (coords[i], coords[i+1]);
                            table[coords[i], coords[i + 1]] = new Barrack() { IsBlue = false };
                            indexRed++;
                        }
                    }

                    table.BlueGold = Int32.Parse(reader.ReadLine());
                    table.BlueHealth = Int32.Parse(reader.ReadLine());

                    table.RedGold = Int32.Parse(reader.ReadLine());
                    table.RedHealth = Int32.Parse(reader.ReadLine());


                    int blueTowers = Int32.Parse(reader.ReadLine());    
                    table.BlueTowerPositions = new List<(int, int)>();
                    if (blueTowers != 0)
                    {
                        for (int i = 0; i<blueTowers; i++)
                        {
                            float[] row = reader.ReadLine().Split(' ').Select(e => float.Parse(e)).ToArray();

                            table.BlueTowerPositions.Add(((int)row[0], (int)row[1]));

                            Tower t;
                            if (row[4] < 3)
                            {
                                t = new SmallTower();
                            } else if ( row[4]< 5)
                            {
                                t = new MediumTower();
                            } else
                            {
                                t = new LargeTower();
                            }
                            t.IsBlue = true;
                            t.Range = row[2];
                            t.Damage = (int)row[3];
                            t.Level = (int)row[4];
                            table[(int)row[0], (int)row[1]] = t;
                        }
                    }


                    int redTowers = Int32.Parse(reader.ReadLine());
                    table.RedTowerPositions = new List<(int, int)>();
                    if (redTowers != 0)
                    {
                        for (int i = 0; i < redTowers; i++)
                        {
                            float[] row = reader.ReadLine().Split(' ').Select(e => float.Parse(e)).ToArray();
                            table.RedTowerPositions.Add(((int)row[0], (int)row[1]));
                            Tower t;
                            if (row[4] < 3)
                            {
                                t = new SmallTower();
                            }
                            else if (row[4] < 5)
                            {
                                t = new MediumTower();
                            }
                            else
                            {
                                t = new LargeTower();
                            }
                            t.IsBlue = false;
                            t.Range = row[2];
                            t.Damage = (int)row[3];
                            t.Level = (int)row[4];
                            table[(int)row[0], (int)row[1]] = t;
                        }
                    }

                    table.BlueUnits = new List<Unit>();
                    table.BlueUnitsCurrentPositions = new List<List<List<Unit>>>();

                    table.RedUnits = new List<Unit>();
                    table.RedUnitsCurrentPositions = new List<List<List<Unit>>>();

                    return table;
                }
            }
            catch
            {
                throw new GameDataException();
            }
        }

        /// <summary>
        /// Save game state to a file
        /// </summary>
        /// <param name="path">Path of the file</param>
        /// <param name="table">Current state of the game</param>
        public async Task SaveAsync(String path, GameTable table)
        {
            try
            {
                using (StreamWriter writer = new StreamWriter(path)) // fájl megnyitása
                {
                    writer.WriteLine(table.Height.ToString() + " " + table.Width.ToString());
                    writer.WriteLine(table.State == GameState.Red ? "red" : (table.State == GameState.Blue ? "blue" : "mapeditor"));

                    StringBuilder sb = new StringBuilder("", table.Width);
                    for (int i = 0; i<table.Height; i++)
                    {
                        for (int j = 0; j<table.Width; j++)
                        {
                            if (table[i, j] is Dune)
                            {
                                sb.Append('D');
                            }
                            else if (table[i, j] is Oasis)
                            {
                                sb.Append('O');
                            }
                            else if (table[i, j] is Ruins)
                            {
                                sb.Append('R');
                            } else
                            {
                                sb.Append('E');
                            }

                            if (j!=table.Width-1)
                            {
                                sb.Append(' ');
                            }
                        }
                        writer.WriteLine(sb.ToString());
                        sb.Clear();
                    }

                    if (table.BlueCastle is null)
                    {
                        writer.WriteLine("0");
                    } else
                    {
                        writer.WriteLine("1");
                        writer.WriteLine(table.BlueCastle.Position.Y.ToString() + " " + table.BlueCastle.Position.X.ToString());
                    }
                    if (table.RedCastle is null)
                    {
                        writer.WriteLine("0");
                    } else
                    {
                        writer.WriteLine("1");
                        writer.WriteLine(table.RedCastle.Position.Y.ToString() + " " + table.RedCastle.Position.X.ToString());
                    }


                    /*
                    //Test
                    table.BlueBarracks[0] = new Tuple<int, int>(1, 0);
                    table.BlueBarracks[1] = new Tuple<int, int>(1, 2);
                    */

                    // Barracks
                    if (/*table.BlueBarracks[0] is null*/table.BlueBarracks is null || table.BlueBarracks.Length == 0) // TODO: kijavítani, ha rossz! -> az (int, int) nem nullable
                    {
                        writer.WriteLine("0");
                    } else
                    {
                        writer.WriteLine(table.BlueBarracks.Length);
                        writer.WriteLine(String.Join(" ", table.BlueBarracks.Select((coordinates) => coordinates.Item1.ToString() + " " + coordinates.Item2.ToString()).ToArray()));
                    }
                    if (/*table.RedBarracks[0] is null*/table.RedBarracks is null || table.RedBarracks.Length == 0) // TODO: kijavítani, ha rossz! -> az (int, int) nem nullable
                    {
                        writer.WriteLine("0");
                    }
                    else
                    {
                        writer.WriteLine(table.RedBarracks.Length);
                        writer.WriteLine(String.Join(" ", table.RedBarracks.Select((coordinates) => coordinates.Item1.ToString() + " " + coordinates.Item2.ToString()).ToArray()));
                    }

                    writer.WriteLine(table.BlueGold);
                    writer.WriteLine(table.BlueHealth);

                    writer.WriteLine(table.RedGold);
                    writer.WriteLine(table.RedHealth);

                    if (table.BlueTowerPositions is null || table.BlueTowerPositions.Count == 0)
                    {
                        writer.WriteLine("0");
                    } else
                    {
                        writer.WriteLine(table.BlueTowerPositions.Count);
                        foreach ((int Y, int X) pos in table.BlueTowerPositions)
                        {
                            // if (table[pos.Item1, pos.Item2] is SmallTower)
                            {
                                Tower t = (Tower)table[pos.Y, pos.X];
                                float[] row = { pos.Y, pos.X, t.Range, t.Damage, t.Level };
                                writer.WriteLine(String.Join(" ",row.Select(e=>e.ToString())));
                            }
                        }
                    }
                    if (table.RedTowerPositions is null || table.RedTowerPositions.Count == 0)
                    {
                        writer.WriteLine("0");
                    }
                    else
                    {
                        writer.WriteLine(table.RedTowerPositions.Count);
                        foreach ((int Y, int X) pos in table.RedTowerPositions)
                        {
                            // if (table[pos.Item1, pos.Item2] is Tower)
                            {
                                Tower t = (Tower)table[pos.Y, pos.X];
                                float[] row = { pos.Y, pos.X, t.Range, t.Damage, t.Level };

                                writer.WriteLine(String.Join(" ", row.Select(e => e.ToString())));
                            }
                        }
                    }

                }
            }
            catch
            {
                throw new GameDataException();
            }
        }
    }
}
