﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Text.RegularExpressions;
using System.Windows.Controls;

namespace ScaraBattle.View
{
    /// <summary>
    /// Interaction logic for SizeDialogBox.xaml
    /// </summary>
    public partial class SizeDialogBox : Window
    {
        public int MapWidth { get; private set; }

        public int MapHeight { get; private set; }

        public int Barracks { get; private set; }

        public int Terrain { get; private set; }

        public int MaxBarrackNum { get; private set; }

        public bool MapEditor { get; set; }

        public SizeDialogBox(bool mapEditor=false)
        {
            InitializeComponent();

            MapEditor = mapEditor;

            if (MapEditor)
            {
                StackPanel extraOptions = FindName("ExtraOptions") as StackPanel;
                extraOptions.Visibility = Visibility.Hidden;

                Barracks = 2;
                UpdateMaxBarrackNumLabel();
            }
        }

        private void UpdateMaxBarrackNumLabel()
        {
            if (!MapEditor)
            {
                Label maxBarrackNumLabel = FindName("MaxBarrackNumLabel") as Label;
                if (maxBarrackNumLabel != null)
                {
                    maxBarrackNumLabel.Content = MaxBarrackNum;
                }
            }
        }

        private void SetMaxBarrackNum()
        {
            MaxBarrackNum = MapWidth * MapHeight < 225 ? 3 : 4;
            UpdateMaxBarrackNumLabel();
        }

        private void TextBox_NumberOnly(object sender, TextCompositionEventArgs e)
        {
            e.Handled = !IsTextAllowed(e.Text);
        }

        private static readonly Regex _regex = new Regex("[^0-9.-]+");
        private static bool IsTextAllowed(string text)
        {
            return !_regex.IsMatch(text);
        }

        private void UpdateWidth(object sender, TextChangedEventArgs e)
        {
            string text = ((TextBox)sender).Text;
            MapWidth =  text == String.Empty ? 0 : Convert.ToInt32(text);

            SetMaxBarrackNum();
        }

        private void UpdateHeight(object sender, TextChangedEventArgs e)
        {
            string text = ((TextBox)sender).Text;
            MapHeight = text == String.Empty ? 0 : Convert.ToInt32(text);

            SetMaxBarrackNum();
        }

        private void UpdateBarracks(object sender, TextChangedEventArgs e)
        {
            string text = ((TextBox)sender).Text;
            Barracks = text == String.Empty ? 0 : Convert.ToInt32(text);
        }

        private void UpdateTerrain(object sender, SelectionChangedEventArgs e)
        {
            switch (((ComboBoxItem)((ComboBox)sender).SelectedItem).Content.ToString())
            {
                case "Síkság":
                    Terrain = 0;
                    break;
                case "Változatos":
                    Terrain = 1;
                    break;
                case "Extrém":
                    Terrain = 2;
                    break;
            }
        }

        private void Button_Click(object Sender, RoutedEventArgs e)
        {
            if (10 <= MapWidth && MapWidth <= 20 &&
                10 <= MapHeight && MapHeight <= 20 &&
                2 <= Barracks && Barracks <= MaxBarrackNum)
            {
                DialogResult = true;
                Close();
            }
            else
            {
                MessageBox.Show("Nincs minden adat az elfogadható intervallumban, válasszon más beállításokat!", "Hiba", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }
    }
}
