﻿using System;
using System.Collections.ObjectModel;
using ScaraBattle.Model;
using ScaraBattle.Persistence;
using System.Windows.Threading;
using System.Collections.Generic;
using System.Diagnostics;

namespace ScaraBattle.ViewModel
{
    public enum InfoType
    {
        Tower,
        General, // üres cella, akadályok
        Unit,
        Nothing,
        Text
    }

    class GameViewModel : ViewModelBase
    {
        #region Fields

        private GameModel _model; // modell
        // semmi
        private const String NothingImage = "NothingImage";
        // obstacles
        private const String DuneImage = "DuneImage";
        private const String OasisImage = "OasisImage";
        private const String RuinsImage = "RuinsImage";
        // castle, barrack
        private const String CastleImage = "CastleImage";
        private const String BarrackImage = "BarrackImage";

        private const String CastleImageBlue = "CastleImageBlue";
        private const String BarrackImageBlue = "BarrackImageBlue";

        private const String CastleImageRed = "CastleImageRed";
        private const String BarrackImageRed = "BarrackImageRed";
        // units
        private const String ScarabImage = "ScarabImage";
        private const String ScarabImageBlue = "ScarabImageBlue";
        private const String ScarabImageRed = "ScarabImageRed";
        private const String ScarabImageMore = "ScarabImageMore";
        // towers
        private const String SmallTowerImage = "SmallTowerImage";
        private const String MediumTowerImage = "MediumTowerImage";
        private const String LargeTowerImage = "LargeTowerImage";

        private const String SmallTowerImageBlue = "SmallTowerImageBlue";
        private const String MediumTowerImageBlue = "MediumTowerImageBlue";
        private const String LargeTowerImageBlue = "LargeTowerImageBlue";

        private const String SmallTowerImageRed = "SmallTowerImageRed";
        private const String MediumTowerImageRed = "MediumTowerImageRed";
        private const String LargeTowerImageRed = "LargeTowerImageRed";

        private int _offsetY;
        private int _offsetX;

        private SelectedObject _selectedObject = SelectedObject.Nothing;

        private (int Y, int X) _selectedCell;

        private DispatcherTimer _timer;

        #endregion

        #region Properties

        /// <summary>
        /// Új játék kezdése parancs lekérdezése.
        /// </summary>
        public DelegateCommand NewGameCommand { get; private set; }

        /// <summary>
        /// Térkép létrehozása parancs lekérdezése.
        /// </summary>
        public DelegateCommand CreateMapCommand { get; private set; }

        /// <summary>
        /// Játék betöltése parancs lekérdezése.
        /// </summary>
        public DelegateCommand LoadGameCommand { get; private set; }

        /// <summary>
        /// Játék mentése parancs lekérdezése.
        /// </summary>
        public DelegateCommand SaveGameCommand { get; private set; }

        /// <summary>
        /// Kilépés parancs lekérdezése.
        /// </summary>
        public DelegateCommand ExitGameCommand { get; private set; }

        /// <summary>
        /// Súgó parancs lekérdezése.
        /// </summary>
        public DelegateCommand HelpCommand { get; private set; }

        /// <summary>
        /// Térképszerkesztő megnyitása parancs lekérdezése.
        /// </summary>
        public DelegateCommand MapEditorCommand { get; private set; }

        /// <summary>
        /// Kamera mozgatása felfelé parancs lekérdezése.
        /// </summary>
        public DelegateCommand CameraUpCommand { get; private set; }

        /// <summary>
        /// Kamera mozgatása balra parancs lekérdezése.
        /// </summary>
        public DelegateCommand CameraLeftCommand { get; private set; }

        /// <summary>
        /// Kamera mozgatása jobbra parancs lekérdezése.
        /// </summary>
        public DelegateCommand CameraRightCommand { get; private set; }

        /// <summary>
        /// Kamera mozgatása lefelé parancs lekérdezése.
        /// </summary>
        public DelegateCommand CameraDownCommand { get; private set; }

        /// <summary>
        /// Torony eladása parancs lekérdezése.
        /// </summary>
        public DelegateCommand SellTowerCommand { get; private set; }

        /// <summary>
        /// Torony fejlesztése parancs lekérdezése.
        /// </summary>
        public DelegateCommand UpgradeTowerCommand { get; private set; }

        /// <summary>
        /// Játékállapot előre léptetése parancs lekérdezése.
        /// </summary>
        public DelegateCommand StepGameStateCommand { get; private set; }

        /// <summary>
        /// Játékmező gyűjtemény lekérdezése.
        /// </summary>
        public ObservableCollection<GameField> Fields { get; set; }

        /// <summary>
        /// Játékmenü gyűjtemény lekérdezése.
        /// </summary>
        public ObservableCollection<MenuItem> GameMenuItems { get; set; }

        /// <summary>
        /// Térképmenü gyűjtemény lekérdezése.
        /// </summary>
        public ObservableCollection<MenuItem> MapEditorMenuItems { get; set; }

        /// <summary>
        /// Egység gyűjtemény lekérdezése.
        /// </summary>
        public ObservableCollection<UnitListItem> UnitListItems { get; set; }

        /// <summary>
        /// A játékterület magassága
        /// </summary>
        public int GridHeight { get; set; }

        /// <summary>
        /// A játékterület szélessége
        /// </summary>
        public int GridWidth { get; set; }

        public string SelectedItem { get; set; }

        public int BlueGold { get { return _model.Table == null ? 0 : _model.Table.BlueGold; } }

        public int RedGold { get { return _model.Table == null ? 0 : _model.Table.RedGold; } }
        public int BlueHealth { get { return _model.Table == null ? 0 : _model.Table.BlueHealth; } }
        public int RedHealth { get { return _model.Table == null ? 0 : _model.Table.RedHealth; } }
        public int BlueSmallUnits { get { return _model.Table == null ? 0 : _model.Table.BlueSmallUnits; } }
        public int BlueLargeUnits { get { return _model.Table == null ? 0 : _model.Table.BlueLargeUnits; } }
        public int RedSmallUnits { get { return _model.Table == null ? 0 : _model.Table.RedSmallUnits; } }
        public int RedLargeUnits { get { return _model.Table == null ? 0 : _model.Table.RedLargeUnits; } }

        public int TowerDamage { get; set; }
        public float TowerRange { get; set; }
        public int TowerLevel { get; set; }
        public int TowerSellCost { get; set; }
        public int TowerUpgradeCost { get; set; }

        public string InfoName { get; set; }

        public string InfoImage { get; set; }

        public string WhyIsNothingHappening { get; set; }

        public InfoType VisibleInfo { get; set; }

        public string StepGameButtonText { get; set; }

        private bool canGoUp;
        public bool CanGoUp
        {
            get { return canGoUp; }
            set { canGoUp = value;
                OnPropertyChanged(nameof(CanGoUp));
            }
        }
        private bool canGoDown;
        public bool CanGoDown
        {
            get { return canGoDown; }
            set
            {
                canGoDown = value;
                OnPropertyChanged(nameof(CanGoDown));
            }
        }
        private bool canGoLeft;
        public bool CanGoLeft
        {
            get { return canGoLeft; }
            set
            {
                canGoLeft = value;
                OnPropertyChanged(nameof(CanGoLeft));
            }
        }
        private bool canGoRight;
        public bool CanGoRight
        {
            get { return canGoRight; }
            set
            {
                canGoRight = value;
                OnPropertyChanged(nameof(CanGoRight));
            }
        }

        public GameState State { get { return _model.Table == null ? GameState.Menu : _model.Table.State; } }

        #endregion

        #region Events

        /// <summary>
        /// Új játék eseménye.
        /// </summary>
        public event EventHandler NewGame;

        /// <summary>
        /// Térkép létrehozásának eseménye.
        /// </summary>
        public event EventHandler CreateMap;

        /// <summary>
        /// Játék betöltésének eseménye.
        /// </summary>
        public event EventHandler LoadGame;

        /// <summary>
        /// Játék mentésének eseménye.
        /// </summary>
        public event EventHandler SaveGame;

        /// <summary>
        /// Játékból való kilépés eseménye.
        /// </summary>
        public event EventHandler ExitGame;

        /// <summary>
        /// Súgó kérésének eseménye.
        /// </summary>
        public event EventHandler Help;

        /// <summary>
        /// Saját térkép szerkesztésének eseménye.
        /// </summary>
        public event EventHandler EditMap;

        /// <summary>
        /// Kamera felfele mozgatásának eseménye.
        /// </summary>
        public event EventHandler CameraUp;

        /// <summary>
        /// Kamera balra mozgatásának eseménye.
        /// </summary>
        public event EventHandler CameraLeft;

        /// <summary>
        /// Kamera jobbra mozgatásának eseménye.
        /// </summary>
        public event EventHandler CameraRight;

        /// <summary>
        /// Kamera lefele mozgatásának eseménye.
        /// </summary>
        public event EventHandler CameraDown;

        private void TimerTick(object sender, EventArgs e)
        {
            if (_model.Table.State == GameState.Simulation)
            {
                _model.Simulate();
            }
            
        }


        #endregion

        #region Constructors

        /// <summary>
        /// Nézetmodell példányosítása.
        /// </summary>
        /// <param name="model">A modell típusa.</param>
        public GameViewModel(GameModel model)
        {
            // játék csatlakoztatása
            _model = model;
            _model.GameAdvanced += new EventHandler<GameEventArgs>(Model_GameAdvanced);
            _model.GameCreated += new EventHandler<GameEventArgs>(Model_GameCreated);
            _model.GameOver += new EventHandler<GameEventArgs>((obj, e) => RefreshTable());

            _timer = new DispatcherTimer();
            _timer.Tick += TimerTick;
            _timer.Interval = new TimeSpan(0, 0, 1);
            _timer.Start();


            // parancsok kezelése
            NewGameCommand = new DelegateCommand(param => OnNewGame());
            LoadGameCommand = new DelegateCommand(param => OnLoadGame());
            SaveGameCommand = new DelegateCommand(param => OnSaveGame());
            ExitGameCommand = new DelegateCommand(param => OnExitGame());
            HelpCommand = new DelegateCommand(param => OnHelpRequested());
            MapEditorCommand = new DelegateCommand(param => OnEditMap());

            CameraUpCommand = new DelegateCommand(param => OnCameraUp());
            CameraLeftCommand = new DelegateCommand(param => OnCameraLeft());
            CameraRightCommand = new DelegateCommand(param => OnCameraRight());
            CameraDownCommand = new DelegateCommand(param => OnCameraDown());

            InitializeMenuItems();

            CreateMapCommand = new DelegateCommand(param => OnCreateMap());

            SellTowerCommand = new DelegateCommand(param => OnSellTower());
            UpgradeTowerCommand = new DelegateCommand(param => OnUpgradeTower());

            StepGameStateCommand = new DelegateCommand(param => OnStepGameState());
        }

        #endregion

        #region Public methods

        public void GenerateTiles()
        {
            GridHeight = 10;
            GridWidth = 10;

            _offsetX = (_model.Table.Width / 2) - (GridWidth/2);
            _offsetY = (_model.Table.Height / 2) - (GridHeight/2);
            CanGoUp = _offsetY > 0;
            CanGoDown = _offsetY < _model.Table.Height - GridHeight;
            CanGoLeft = _offsetX > 0;
            CanGoRight = _offsetX < _model.Table.Width - GridWidth;

            Fields = new ObservableCollection<GameField>();
            for (int i = 0; i < GridHeight; i++)
            {
                for (int j = 0; j < GridWidth; j++)
                {
                    Fields.Add(new GameField
                    {
                        X = i,
                        Y = j,
                        Image = "",
                        CellCommand = new DelegateCommand(param => SelectCell((((int Y, int X))param).Y, (((int Y, int X))param).X))
                    });
                }
            }

            RefreshTable();
        }

        /// <summary>
        /// Tábla frissítése.
        /// </summary>
        public void RefreshTable()
        {
            foreach (GameField field in Fields)
            {
                field.Image = "";
            }

            for (int i = 0; i < GridHeight; ++i)
            {
                for (int j = 0; j < GridWidth; ++j)
                {
                    // _model.Table[i, j]) alapján beállítani a mező képét
                    object cell = _model.Table[i + _offsetY, j + _offsetX];

                    if (cell is Dune)
                    {
                        GetField(i, j).Image = DuneImage;
                    }
                    else if (cell is Oasis)
                    {
                        GetField(i, j).Image = OasisImage;
                    }
                    else if (cell is Ruins)
                    {
                        GetField(i, j).Image = RuinsImage;
                    }
                    else if (cell is Castle c)
                    {
                        if (c.IsBlue)
                        {
                          GetField(i, j).Image = CastleImageBlue;
                        }
                        else {
                            GetField(i, j).Image = CastleImageRed;
                        }
                    }
                    else if (cell is Barrack b) // ennek is van isblueja?
                    {
                        if (b.IsBlue)
                        {
                            GetField(i, j).Image = BarrackImageBlue;
                        }
                        else {
                            GetField(i, j).Image = BarrackImageRed;
                        }
                    }
                }
            }

            if (_model.Table.State != GameState.MapEditor)
            {
                // Move units
                if (_model.Table.State == GameState.Simulation)
                {
                    // Blue units
                    for (int i = 0; i<_model.Table.BlueUnitsCurrentNum; i++)
                    {
                        int y = _model.Table.BlueUnits[i].Y;
                        int x = _model.Table.BlueUnits[i].X;
                        if (0 <= x - _offsetX && x - _offsetX < GridWidth &&
                            0 <= y - _offsetY && y - _offsetY < GridHeight)
                        {
                            if (_model.IsTwoUnits(y,x))
                            {
                                GetField(y - _offsetY, x - _offsetX).Image = ScarabImageMore;
                            }
                            else if (_model.Table.BlueUnits[i] is LargeUnit)
                            {
                                GetField(y - _offsetY, x - _offsetX).Image = ScarabImageBlue;
                            }
                            else
                            {
                                GetField(y - _offsetY, x - _offsetX).Image = ScarabImageBlue;
                            }
                        }
                    }

                    // Red units
                    for (int i = 0; i < _model.Table.RedUnitsCurrentNum; i++)
                    {
                        int y = _model.Table.RedUnits[i].Y;
                        int x = _model.Table.RedUnits[i].X;
                        if (0 <= x - _offsetX && x - _offsetX < GridWidth &&
                            0 <= y - _offsetY && y - _offsetY < GridHeight)
                        {
                            if (_model.IsTwoUnits(y, x))
                            {
                                GetField(y - _offsetY, x - _offsetX).Image = ScarabImageMore;
                            }
                            else if (_model.Table.RedUnits[i] is LargeUnit)
                            {
                                GetField(y - _offsetY, x - _offsetX).Image = ScarabImageRed;
                            }
                            else
                            {
                                GetField(y - _offsetY, x - _offsetX).Image = ScarabImageRed;
                            }
                        }
                    }
                }

                foreach ((int Y, int X) t in _model.Table.BlueTowerPositions)
                {
                    int y = t.Y;
                    int x = t.X;

                    if (0 <= x - _offsetX && x - _offsetX < GridWidth &&
                        0 <= y - _offsetY && y - _offsetY < GridHeight)
                    {
                        object cell = _model.Table[y, x];

                        if (cell is SmallTower)
                        {
                            GetField(y - _offsetY, x - _offsetX).Image = SmallTowerImageBlue;
                        }
                        else if (cell is MediumTower)
                        {
                            GetField(y - _offsetY, x - _offsetX).Image = MediumTowerImageBlue;
                        }
                        else if (cell is LargeTower)
                        {
                            GetField(y - _offsetY, x - _offsetX).Image = LargeTowerImageBlue;
                        }
                    }
                }

                foreach ((int Y, int X) t in _model.Table.RedTowerPositions)
                {
                    int y = t.Y;
                    int x = t.X;

                    if (0 <= x - _offsetX && x - _offsetX < GridWidth &&
                        0 <= y - _offsetY && y - _offsetY < GridHeight)
                    {
                        object cell = _model.Table[y, x];

                        if (cell is SmallTower)
                        {
                            GetField(y - _offsetY, x - _offsetX).Image = SmallTowerImageRed;
                        }
                        else if (cell is MediumTower)
                        {
                            GetField(y - _offsetY, x - _offsetX).Image = MediumTowerImageRed;
                        }
                        else if (cell is LargeTower)
                        {
                            GetField(y - _offsetY, x - _offsetX).Image = LargeTowerImageRed;
                        }
                    }
                }
            }

            OnPropertyChanged(nameof(Fields));
            OnPropertyChanged(nameof(SelectedItem));
            OnPropertyChanged(nameof(BlueGold));
            OnPropertyChanged(nameof(RedGold));
            OnPropertyChanged(nameof(BlueHealth));
            OnPropertyChanged(nameof(RedHealth));
            OnPropertyChanged(nameof(BlueSmallUnits));
            OnPropertyChanged(nameof(BlueLargeUnits));
            OnPropertyChanged(nameof(RedSmallUnits));
            OnPropertyChanged(nameof(RedLargeUnits));
            OnPropertyChanged(nameof(State));
        }

        #endregion

        #region Private methods
        private void InitializeMenuItems()
        {
            InitializeGameMenuItems();
            InitializeMapEditorMenuItems();
        }

        private void InitializeGameMenuItems()
        {
            GameMenuItems = new ObservableCollection<MenuItem>();

            // Kis torony
            GameMenuItems.Add(new MenuItem()
            {
                Image = SmallTowerImage,
                Type = SelectedObject.SmallTower,
                Price = SmallTower.Price,
                SelectCommand = new DelegateCommand(param => OnSelectMenuItem((SelectedObject)param))
            });

            // Közepes torony
            GameMenuItems.Add(new MenuItem()
            {
                Image = MediumTowerImage,
                Type = SelectedObject.MediumTower,
                Price = MediumTower.Price,
                SelectCommand = new DelegateCommand(param => OnSelectMenuItem((SelectedObject)param))
            });

            // Nagy torony
            GameMenuItems.Add(new MenuItem()
            {
                Image = LargeTowerImage,
                Type = SelectedObject.LargeTower,
                Price = LargeTower.Price,
                SelectCommand = new DelegateCommand(param => OnSelectMenuItem((SelectedObject)param))
            });

            // Kis egység
            GameMenuItems.Add(new MenuItem()
            {
                Image = ScarabImage,
                Type = SelectedObject.SmallUnit,
                Price = SmallUnit.Price,
                SelectCommand = new DelegateCommand(param => OnSelectMenuItem((SelectedObject)param))
            });

            // Nagy egység
            GameMenuItems.Add(new MenuItem()
            {
                Image = ScarabImage,
                Type = SelectedObject.LargeUnit,
                Price = LargeUnit.Price,
                SelectCommand = new DelegateCommand(param => OnSelectMenuItem((SelectedObject)param))
            });
        }

        private void InitializeMapEditorMenuItems()
        {
            MapEditorMenuItems = new ObservableCollection<MenuItem>();

            // Dűne
            MapEditorMenuItems.Add(new MenuItem()
            {
                Image = DuneImage,
                Type = SelectedObject.Dune,
                SelectCommand = new DelegateCommand(param => OnSelectMenuItem((SelectedObject)param))
            });

            // Oázis
            MapEditorMenuItems.Add(new MenuItem()
            {
                Image = OasisImage,
                Type = SelectedObject.Oasis,
                SelectCommand = new DelegateCommand(param => OnSelectMenuItem((SelectedObject)param))
            });

            // Rom
            MapEditorMenuItems.Add(new MenuItem()
            {
                Image = RuinsImage,
                Type = SelectedObject.Ruins,
                SelectCommand = new DelegateCommand(param => OnSelectMenuItem((SelectedObject)param))
            });

            // Kastély
            MapEditorMenuItems.Add(new MenuItem()
            {
                Image = CastleImage,
                Type = SelectedObject.Castle,
                SelectCommand = new DelegateCommand(param => OnSelectMenuItem((SelectedObject)param))
            });

            // Barakk
            MapEditorMenuItems.Add(new MenuItem()
            {
                Image = BarrackImage,
                Type = SelectedObject.Barrack,
                SelectCommand = new DelegateCommand(param => OnSelectMenuItem((SelectedObject)param))
            });
        }

        /// <summary>
        /// Cella lekérése pozíció alapján
        /// </summary>
        private GameField GetField(int i, int j)
        {
            return Fields[i * GridWidth + j];
        }

        private void ShowSelectedTower(int y, int x) // sima y-nal és x-szel kell meghívni offset nélkül, azt kiszámolja magának
        {
            object cell = _model.Table[y + _offsetY, x + _offsetX];

            // információ megjelenítése alul
            if (cell is Tower)
            {
                _selectedCell = (y + _offsetY, x + _offsetX);

                TowerDamage = ((Tower)cell).Damage;
                float ize = ((Tower)cell).Range;
                TowerRange = (float)Math.Round(ize, 2);
                TowerLevel = ((Tower)cell).Level;
                TowerSellCost = ((Tower)cell).SellCost;
                TowerUpgradeCost = ((Tower)cell).UpgradeCost;
                VisibleInfo = InfoType.Tower;

                if (cell is SmallTower t)
                {
                    InfoName = "Kis torony";
                    InfoImage = t.IsBlue ? SmallTowerImageBlue : SmallTowerImageRed;
                }
                else if (cell is MediumTower m)
                {
                    InfoName = "Közepes torony";
                    InfoImage = m.IsBlue ? MediumTowerImageBlue : MediumTowerImageRed;
                }
                else if (cell is LargeTower l)
                {
                    InfoName = "Nagy torony";
                    InfoImage = l.IsBlue ? LargeTowerImageBlue : LargeTowerImageRed;
                }

                OnPropertyChanged(nameof(TowerDamage));
                OnPropertyChanged(nameof(TowerRange));
                OnPropertyChanged(nameof(TowerLevel));
                OnPropertyChanged(nameof(TowerSellCost));
                OnPropertyChanged(nameof(TowerUpgradeCost));
                OnPropertyChanged(nameof(VisibleInfo));
                OnPropertyChanged(nameof(InfoName));
                OnPropertyChanged(nameof(InfoImage));
            }
        }
        
        private void SelectCell(int y, int x)
        {
            if (_model.Table.State == GameState.MapEditor)
            {
                _model.PlaceObject(y + _offsetY, x + _offsetX, _selectedObject);
            }
            else if (_model.Table.State == GameState.Simulation)
            {
                List<Unit> units = new List<Unit>();

                foreach (Unit u in _model.Table.BlueUnits)
                {
                    if (u.Y == y + _offsetX && u.X == x + _offsetX)
                    {
                        units.Add(u);
                    }
                }

                foreach (Unit u in _model.Table.RedUnits)
                {
                    if (u.Y == y + _offsetX && u.X == x + _offsetX)
                    {
                        units.Add(u);
                    }
                }

                if (units.Count > 0)
                {
                    VisibleInfo = InfoType.Unit;
                    UnitListItems = new ObservableCollection<UnitListItem>();

                    foreach (Unit u in units)
                    {
                        SelectedObject unitType = SelectedObject.Nothing;
                        if (u is SmallUnit)
                        {
                            unitType = SelectedObject.SmallUnit;
                        }
                        else if (u is LargeUnit)
                        {
                            unitType = SelectedObject.LargeUnit;
                        }

                        UnitListItems.Add(new UnitListItem() { Health = u.Health, Type = unitType, IsBlue = u.IsBlue });
                    }

                    OnPropertyChanged(nameof(VisibleInfo));
                    OnPropertyChanged(nameof(UnitListItems));
                }
                else
                {
                    VisibleInfo = InfoType.Nothing;
                    OnPropertyChanged(nameof(VisibleInfo));
                    return;
                }
            }
            else if (_selectedObject != SelectedObject.Nothing) // valamit le akarunk rakni játékon belül (tornyot)
            {
                _model.PlaceObject(y + _offsetY, x + _offsetX, _selectedObject);
                ShowSelectedTower(y, x); // ha leraktuk a tornyot, ki is választódik
            }
            else
            {
                object cell = _model.Table[y + _offsetY, x + _offsetX];

                // információ megjelenítése alul
                if (cell is Tower)
                {
                    ShowSelectedTower(y, x);
                }

                else if (cell is Obstacle || cell is Castle || cell is Barrack)
                {
                    _selectedCell = (y + _offsetY, x + _offsetX);

                    VisibleInfo = InfoType.General;

                    if (cell is Dune)
                    {
                        InfoName = "Dűne";
                        InfoImage = "DuneImage";
                    }
                    if (cell is Oasis)
                    {
                        InfoName = "Oázis";
                        InfoImage = "OasisImage";
                    }
                    if (cell is Ruins)
                    {
                        InfoName = "Rom";
                        InfoImage = "RuinsImage";
                    }
                    if (cell is Castle c)
                    {
                        InfoName = "Kastély";
                        InfoImage = c.IsBlue ? CastleImageBlue : CastleImageRed;
                        //InfoImage = "CastleImage";
                    }
                    if (cell is Barrack b)
                    {
                        InfoName = "Barakk";
                        InfoImage = b.IsBlue ? BarrackImageBlue : BarrackImageRed;
                        //InfoImage = "BarrackImage";
                    };
                    OnPropertyChanged(nameof(VisibleInfo));
                    OnPropertyChanged(nameof(InfoName));
                    OnPropertyChanged(nameof(InfoImage));
                }
                else
                {
                    _selectedCell = (y + _offsetY, x + _offsetX);

                    VisibleInfo = InfoType.General;

                    //_selectedCell = null;
                    InfoName = "Üres";
                    //InfoImage = "";
                    InfoImage = NothingImage;

                    OnPropertyChanged(nameof(VisibleInfo));
                    OnPropertyChanged(nameof(InfoName));
                    OnPropertyChanged(nameof(InfoImage));
                }
            }


            RefreshTable();
        }

        private void SetStepGameButtonText()
        {
            switch (_model.Table.State)
            {
                case GameState.Blue:
                    StepGameButtonText = "Kör átadása";
                    break;
                case GameState.Red:
                    StepGameButtonText = "Támadás indítása";
                    break;
                case GameState.Simulation:
                    StepGameButtonText = "Támadás";
                    break;
            }

            OnPropertyChanged(nameof(StepGameButtonText));
        }

        #endregion

        #region Game event handlers

        /// <summary>
        /// Játék létrehozásának eseménykezelője.
        /// </summary>
        private void Model_GameCreated(object sender, GameEventArgs e)
        {
            VisibleInfo = InfoType.Nothing;
            OnPropertyChanged(nameof(VisibleInfo));
            _selectedObject = SelectedObject.Nothing;

            if (_model.Table.State == GameState.MapEditor)
            {
                SelectedItem = "Üres cella";
            }
            else
            {
                SelectedItem = "-";
            }
            OnPropertyChanged(nameof(SelectedItem));

            SetStepGameButtonText();

            GenerateTiles();
        }

        /// <summary>
        /// Játék előrehaladásának eseménykezelője.
        /// </summary>
        private void Model_GameAdvanced(object sender, GameEventArgs e)
        {
            if (e.SoldTower)
            {
                VisibleInfo = InfoType.Nothing; // miután eladtuk már nincs mit megjeleníteni
                _selectedCell = (-1, -1);

                OnPropertyChanged(nameof(VisibleInfo));
            }

            if (e.UpgradedTower)
            {
                SelectCell(e.SelectedCellY - _offsetY, e.SelectedCellX - _offsetX);
            }

            if (e.WhyIsNothingHappening != null)
            {
                // üzenet megjelenítése
                VisibleInfo = InfoType.Text;
                WhyIsNothingHappening = e.WhyIsNothingHappening;
                
                OnPropertyChanged(nameof(VisibleInfo));
                OnPropertyChanged(nameof(WhyIsNothingHappening));
            }

            SetStepGameButtonText();

            RefreshTable();
        }



        #endregion

        #region Event methods

        /// <summary>
        /// Új játék indításának eseménykiváltása.
        /// </summary>
        private void OnNewGame()
        {
            if (NewGame != null)
                NewGame(this, EventArgs.Empty);
        }

        /// <summary>
        /// Térkép létrehozásának eseménykiváltása.
        /// </summary>
        private void OnCreateMap()
        {
            if (CreateMap != null)
                CreateMap(this, EventArgs.Empty);
        }

        /// <summary>
        /// Játék betöltése eseménykiváltása.
        /// </summary>
        private void OnLoadGame()
        {
            if (LoadGame != null)
                LoadGame(this, EventArgs.Empty);
        }

        /// <summary>
        /// Játék mentése eseménykiváltása.
        /// </summary>
        private void OnSaveGame()
        {
            if (SaveGame != null)
                SaveGame(this, EventArgs.Empty);
        }

        /// <summary>
        /// Játékból való kilépés eseménykiváltása.
        /// </summary>
        private void OnExitGame()
        {
            if (ExitGame != null)
                ExitGame(this, EventArgs.Empty);
        }

        /// <summary>
        /// Súgó megnyitásának eseménykiváltása.
        /// </summary>
        private void OnHelpRequested()
        {
            if (Help != null)
                Help(this, EventArgs.Empty);
        }

        /// <summary>
        /// Játék szünetelésének eseménykiváltása.
        /// </summary>
        private void OnEditMap()
        {
            if (EditMap != null)
                EditMap(this, EventArgs.Empty);
        }

        /// <summary>
        /// Kamera felfele léptetésének eseménykiváltása.
        /// </summary>
        public void OnCameraUp()
        {
            if (_offsetY > 0)
            {
                --_offsetY;
                if (_offsetY == 0)
                {
                    CanGoUp = false;
                }
                CanGoDown = true;
                OnPropertyChanged(nameof(CanGoUp));
                OnPropertyChanged(nameof(CanGoDown));
                RefreshTable();
            }
        }

        /// <summary>
        /// Kamera balra léptetésének eseménykiváltása.
        /// </summary>
        public void OnCameraLeft()
        {
            if (_offsetX > 0)
            {
                --_offsetX;
                if (_offsetX == 0)
                {
                    CanGoLeft = false;
                }
                CanGoRight = true;
                OnPropertyChanged(nameof(CanGoLeft));
                OnPropertyChanged(nameof(CanGoRight));
                RefreshTable();
            }
        }

        /// <summary>
        /// Kamera jobbra léptetésének eseménykiváltása.
        /// </summary>
        public void OnCameraRight()
        {
            if (_offsetX < _model.Table.Width - GridWidth)
            {
                ++_offsetX;
                if (_offsetX == _model.Table.Width - GridWidth)
                {
                    CanGoRight = false;
                }
                CanGoLeft = true;
                OnPropertyChanged(nameof(CanGoRight));
                OnPropertyChanged(nameof(CanGoLeft));
                RefreshTable();
            }
        }

        /// <summary>
        /// Kamera lefele léptetésének eseménykiváltása.
        /// </summary>
        public void OnCameraDown()
        {
            if (_offsetY < _model.Table.Height - GridHeight)
            {
                ++_offsetY;
                if (_offsetY == _model.Table.Height - GridHeight)
                {
                    CanGoDown = false;
                }
                CanGoUp = true;
                OnPropertyChanged(nameof(CanGoDown));
                OnPropertyChanged(nameof(CanGoUp));
                RefreshTable();
            }
        }

        private void OnSelectMenuItem(SelectedObject menuItem)
        {
            if (menuItem == SelectedObject.SmallUnit || menuItem == SelectedObject.LargeUnit)
            {
                _model.BuyUnit(menuItem);
                _selectedObject = SelectedObject.Nothing;
            }
            else if (_selectedObject == menuItem)
            {
                SelectedItem = _model.Table.State == GameState.MapEditor ? "Üres cella" : "-";
                _selectedObject = SelectedObject.Nothing;
            }
            else
            {
                _selectedObject = menuItem;

                switch (menuItem)
                {
                    case SelectedObject.SmallTower:
                        SelectedItem = "Kis torony";
                        break;
                    case SelectedObject.MediumTower:
                        SelectedItem = "Közepes torony";
                        break;
                    case SelectedObject.LargeTower:
                        SelectedItem = "Nagy torony";
                        break;
                    case SelectedObject.Dune:
                        SelectedItem = "Dűne";
                        break;
                    case SelectedObject.Oasis:
                        SelectedItem = "Oázis";
                        break;
                    case SelectedObject.Ruins:
                        SelectedItem = "Rom";
                        break;
                    case SelectedObject.Castle:
                        SelectedItem = "Kastély";
                        break;
                    case SelectedObject.Barrack:
                        SelectedItem = "Barakk";
                        break;
                }
            }

            OnPropertyChanged(nameof(SelectedItem));
        }

        /// <summary>
        /// Torony eladásának eseménykiváltása.
        /// </summary>
        private void OnSellTower()
        {
            if (_selectedCell != (-1, -1))
            {
                _model.SellTower(_selectedCell.Y, _selectedCell.X);
            }
        }

        /// <summary>
        /// Torony fejlesztésének eseménykiváltása.
        /// </summary>
        private void OnUpgradeTower()
        {
            if (_selectedCell != (-1, -1))
            {
                _model.UpgradeTower(_selectedCell.Y, _selectedCell.X);

                //object cell = _model.Table[_selectedCell.Y - _offsetY, _selectedCell.X - _offsetX]; //eredeti
                object cell = _model.Table[_selectedCell.Y, _selectedCell.X ];

                TowerDamage = ((Tower)cell).Damage;
                TowerRange = (float)Math.Round(((Tower)cell).Range, 2);
                TowerLevel = ((Tower)cell).Level;
                TowerSellCost = ((Tower)cell).SellCost;
                TowerUpgradeCost = ((Tower)cell).UpgradeCost;

                if (cell is SmallTower t)
                {
                    InfoName = "Kis torony";
                    InfoImage = t.IsBlue ? SmallTowerImageBlue : SmallTowerImageRed;
                }
                else if (cell is MediumTower m)
                {
                    InfoName = "Közepes torony";
                    InfoImage = m.IsBlue ? MediumTowerImageBlue : MediumTowerImageRed;
                }
                else if (cell is LargeTower l)
                {
                    InfoName = "Nagy torony";
                    InfoImage = l.IsBlue ? LargeTowerImageBlue : LargeTowerImageRed;
                }

                OnPropertyChanged(nameof(InfoImage));
                OnPropertyChanged(nameof(TowerDamage));
                OnPropertyChanged(nameof(TowerRange));
                OnPropertyChanged(nameof(TowerLevel));
                OnPropertyChanged(nameof(TowerSellCost));
                OnPropertyChanged(nameof(TowerUpgradeCost));
                OnPropertyChanged(nameof(InfoName));
                OnPropertyChanged(nameof(InfoImage));
            }
        }

        /// <summary>
        /// Játékállapot előreléptetése eseménykiváltása.
        /// </summary>
        private void OnStepGameState()
        {
            _model.StepGameState();
        }        

        #endregion
    }
}
